# -*- coding: utf-8 -*-
#################################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2012 Julius Network Solutions SARL <contact@julius.fr>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#################################################################################
import time
import netsvc
from datetime import date, datetime, timedelta

from osv import fields, osv
from tools import config
from tools.translate import _

class hr_payslip(osv.osv):
    _inherit = 'hr.payslip'
    _columns = {
        'group_line': fields.boolean('Group lines'),
    }
    
    _defaults = {
        'group_line': lambda *a: True,
    }

    def process_sheet(self, cr, uid, ids, context=None):
        move_pool = self.pool.get('account.move')
        move_line_pool = self.pool.get('account.move.line')
        timenow = time.strftime('%Y-%m-%d')
        res = super(hr_payslip, self).process_sheet(cr, uid, ids, context=context)

        for slip in self.browse(cr, uid, ids, context=context):
            account_ids = {}
            if slip.group_line:
                move = slip.move_id
                partner_id = False
                journal_id = slip.journal_id.id
                period_id = slip.period_id.id
                if move:
                    line_ids = move.line_id
                    for line in line_ids:
                        if line.account_id.id not in account_ids.keys():
                            account_ids[line.account_id.id] = {}
                        if line.analytic_account_id and line.analytic_account_id.id not in account_ids[line.account_id.id].keys():
                            account_ids[line.account_id.id][line.analytic_account_id.id] = {}
                            if line.tax_code_id and line.tax_code_id.id not in account_ids[line.account_id.id][line.analytic_account_id.id].keys():
                                account_ids[line.account_id.id][line.analytic_account_id.id][line.tax_code_id.id] = {'name': '', 'total': 0, 'tax_amount': 0}
                            elif not line.tax_code_id and 0 not in account_ids[line.account_id.id][line.analytic_account_id.id].keys():
                                account_ids[line.account_id.id][line.analytic_account_id.id][0] =  {'name': '', 'total': 0, 'tax_amount': 0}
                        elif not line.analytic_account_id:
                            if 0 not in account_ids[line.account_id.id].keys():
                                account_ids[line.account_id.id][0] = {}
                            if line.tax_code_id and line.tax_code_id.id not in account_ids[line.account_id.id][0].keys():
                                account_ids[line.account_id.id][0][line.tax_code_id.id] =  {'name': '', 'total': 0, 'tax_amount': 0}
                            elif not line.tax_code_id and 0 not in account_ids[line.account_id.id][0].keys():
                                account_ids[line.account_id.id][0][0] =  {'name': '', 'total': 0, 'tax_amount': 0}
                        name = account_ids[line.account_id.id][line.analytic_account_id.id or 0][line.tax_code_id.id or 0]['name'] + line.name
                        total = account_ids[line.account_id.id][line.analytic_account_id.id or 0][line.tax_code_id.id or 0]['total'] + line.credit - line.debit
                        tax_amount = account_ids[line.account_id.id][line.analytic_account_id.id or 0][line.tax_code_id.id or 0]['tax_amount'] + line.tax_amount
                        account_ids[line.account_id.id][line.analytic_account_id.id or 0][line.tax_code_id.id or 0] = {'name': name[:63], 'total': total, 'tax_amount': tax_amount}
                    move_pool.button_cancel(cr, uid, [move.id], context=context)
                    for account_id in account_ids.keys():
                        for analytic_id in account_ids[account_id].keys():
                            for tax_id in account_ids[account_id][analytic_id].keys():
                                vals = {
                                    'name': account_ids[account_id][analytic_id][tax_id]['name'],
                                    'date': timenow,
                                    'partner_id': partner_id,
                                    'account_id': account_id or False,
                                    'journal_id': journal_id,
                                    'period_id': period_id,
                                    'debit': account_ids[account_id][analytic_id][tax_id]['total'] < 0.0 and -account_ids[account_id][analytic_id][tax_id]['total'] or 0.0,
                                    'credit': account_ids[account_id][analytic_id][tax_id]['total'] > 0.0 and account_ids[account_id][analytic_id][tax_id]['total'] or 0.0,
                                    'analytic_account_id': analytic_id or False,
                                    'tax_code_id': tax_id or False,
                                    'tax_amount': account_ids[account_id][analytic_id][tax_id]['tax_amount'] or 0.0,
                                    'move_id': move.id,
                                }
                                move_line_pool.create(cr, uid, vals, context=context)
                    unlink_line_ids = [x.id for x in line_ids]
                    move_line_pool.unlink(cr, uid, unlink_line_ids, context=context)
                    if slip.journal_id.entry_posted:
                        move_pool.post(cr, uid, [move.id], context=context)
        return res

hr_payslip()

class account_journal(osv.osv):
    _inherit = 'account.journal'
    _columns = {
        'group_line': fields.boolean('Group lines'),
    }
account_journal()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
