#-*- coding:utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    d$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields
import time
from datetime import datetime
from tools.translate import _

class hr_payslip(osv.osv):
    _inherit = 'hr.payslip'
    
    def process_sheet(self, cr, uid, ids, context=None):    
        move_pool = self.pool.get('account.move')
        period_pool = self.pool.get('account.period')
        timenow = time.strftime('%Y-%m-%d')
        for slip in self.browse(cr, uid, ids, context=context):
            
            # Adjust process for analytic ventilation
            project_ids = {}
            global_info = {'worked_day':0, 'amount':0.0}
            for worked_day_line in slip.worked_days_line_ids:
                global_info['worked_day'] += worked_day_line.number_of_days
                if worked_day_line.account_analytic_id.id not in project_ids.keys():
                    project_ids[worked_day_line.account_analytic_id.id] = 0
                    project_ids[worked_day_line.account_analytic_id.id] = {'days': 0,
                                                                           'amount':0.0,
                                                                           }
                project_ids[worked_day_line.account_analytic_id.id]['days'] += worked_day_line.number_of_days
            # End of adjust Process
            
            line_ids = []            
            debit_sum = 0.0
            credit_sum = 0.0
            if not slip.period_id:
                search_periods = period_pool.find(cr, uid, slip.date_to, context=context)
                period_id = search_periods[0]
            else:
                period_id = slip.period_id.id

            name = _('Payslip of %s') % (slip.employee_id.name)
            move = {
                'narration': name,
                'date': timenow,
                'ref': slip.number,
                'journal_id': slip.journal_id.id,
                'period_id': period_id,
            }
            for line in slip.line_ids:
                
                amt = slip.credit_note and -line.total or line.total
                partner_id = False
                debit_account_id = line.salary_rule_id.account_debit.id
                credit_account_id = line.salary_rule_id.account_credit.id
                
                # Adjust process for analytic ventilation
                split = False
                if line.account_analytic_id:
                    global_info['amount'] += amt
                    account_analytic_id = line.account_analytic_id.id
                    project_ids[line.account_analytic_id.id]['amount'] += amt
                elif not line.salary_rule_id.analytic_account_id:
                    split = True
                    global_info['amount'] += amt
                    for project_id in project_ids:
                        amt_2 = amt*project_ids[project_id]['days']/global_info['worked_day']
                        project_ids[project_id]['amount'] += amt_2
                        if debit_account_id:   
                            debit_line = (0, 0, {
                            'name': line.name,
                            'date': timenow,
                            'partner_id': partner_id,
                            'account_id': debit_account_id,
                            'journal_id': slip.journal_id.id,
                            'period_id': period_id,
                            'debit': amt_2 > 0.0 and amt_2 or 0.0,
                            'credit': amt_2 < 0.0 and -amt_2 or 0.0,
                            'analytic_account_id': project_id,
                            'tax_code_id': line.salary_rule_id.account_tax_id and line.salary_rule_id.account_tax_id.id or False,
                            'tax_amount': line.salary_rule_id.account_tax_id and amt_2 or 0.0,
                            })
                            line_ids.append(debit_line)
                            debit_sum += debit_line[2]['debit'] - debit_line[2]['credit']
                        
                        if credit_account_id:
                            credit_line = (0, 0, {
                            'name': line.name,
                            'date': timenow,
                            'partner_id': partner_id,
                            'account_id': credit_account_id,
                            'journal_id': slip.journal_id.id,
                            'period_id': period_id,
                            'debit': amt_2 < 0.0 and -amt_2 or 0.0,
                            'credit': amt_2 > 0.0 and amt_2 or 0.0,
                            'analytic_account_id': project_id,
                            'tax_code_id': line.salary_rule_id.account_tax_id and line.salary_rule_id.account_tax_id.id or False,
                            'tax_amount': line.salary_rule_id.account_tax_id and amt_2 or 0.0,
                            })
                            line_ids.append(credit_line)
                            credit_sum += credit_line[2]['credit'] - credit_line[2]['debit']
                else:    
                    account_analytic_id = line.salary_rule_id.analytic_account_id and line.salary_rule_id.analytic_account_id.id or False
                # End of adjust Process

                if debit_account_id and not split:   
                    debit_line = (0, 0, {
                    'name': line.name,
                    'date': timenow,
                    'partner_id': partner_id,
                    'account_id': debit_account_id,
                    'journal_id': slip.journal_id.id,
                    'period_id': period_id,
                    'debit': amt > 0.0 and amt or 0.0,
                    'credit': amt < 0.0 and -amt or 0.0,
                    'analytic_account_id': account_analytic_id,
                    'tax_code_id': line.salary_rule_id.account_tax_id and line.salary_rule_id.account_tax_id.id or False,
                    'tax_amount': line.salary_rule_id.account_tax_id and amt or 0.0,
                    })
                    line_ids.append(debit_line)
                    debit_sum += debit_line[2]['debit'] - debit_line[2]['credit']
                 
                if credit_account_id and not split:
                    credit_line = (0, 0, {
                    'name': line.name,
                    'date': timenow,
                    'partner_id': partner_id,
                    'account_id': credit_account_id,
                    'journal_id': slip.journal_id.id,
                    'period_id': period_id,
                    'debit': amt < 0.0 and -amt or 0.0,
                    'credit': amt > 0.0 and amt or 0.0,
                    'analytic_account_id': account_analytic_id,
                    'tax_code_id': line.salary_rule_id.account_tax_id and line.salary_rule_id.account_tax_id.id or False,
                    'tax_amount': line.salary_rule_id.account_tax_id and amt or 0.0,
                    })
                    line_ids.append(credit_line)
                    credit_sum += credit_line[2]['credit'] - credit_line[2]['debit']
            
            if debit_sum > credit_sum:
                acc_id = slip.journal_id.default_credit_account_id.id
                if not acc_id:
                    raise osv.except_osv(_('Configuration Error!'),_('The Expense Journal "%s" has not properly configured the Credit Account!')%(slip.journal_id.name))
                adjust_credit = (0, 0, {
                    'name': _('Adjustment Entry'),
                    'date': timenow,
                    'partner_id': partner_id,
                    'account_id': acc_id,
                    'journal_id': slip.journal_id.id,
                    'period_id': period_id,
                    'debit': 0.0,
                    'credit': debit_sum - credit_sum,
                })
                line_ids.append(adjust_credit)
        
            elif debit_sum < credit_sum:
                acc_id = slip.journal_id.default_debit_account_id.id
                if not acc_id:
                    raise osv.except_osv(_('Configuration Error!'),_('The Expense Journal "%s" has not properly configured the Debit Account!')%(slip.journal_id.name))
                adjust_debit = (0, 0, {
                    'name': _('Adjustment Entry'),
                    'date': timenow,
                    'partner_id': partner_id,
                    'account_id': acc_id,
                    'journal_id': slip.journal_id.id,
                    'period_id': period_id,
                    'debit': credit_sum - debit_sum,
                    'credit': 0.0,
                })
                line_ids.append(adjust_debit)
            move.update({'line_id': line_ids})
            move_id = move_pool.create(cr, uid, move, context=context)
            self.write(cr, uid, [slip.id], {'move_id': move_id, 'period_id' : period_id}, context=context)
            if slip.journal_id.entry_posted:
                move_pool.post(cr, uid, [move_id], context=context)
        
        return self.write(cr, uid, ids, {'paid': True, 'state': 'done'}, context=context)
    
hr_payslip()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
