#-*- coding:utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    d$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
from datetime import datetime
import decimal_precision as dp
from tools.translate import _

SELECTION_LIST = [
    ('csg', 'CSG'),
    ('brut', 'Gross salary'),
    ('smic', 'SMIC'),
    ('more_than_smic', 'Over SMIC'),
    ('ss', 'Social security base'),
    ('ss4', '4 X Social security base'),
    ('sup', 'Overtime base'),
    ('normal', 'Normal hours'),
    ('expense', 'Expense'),
    ('indemnity', 'Indemnity'),
    ('indemnity_gross', 'Indemnity on Gross'),
    ('paid_holiday', 'Paid Holiday'),
    ('advance', 'Advance')
]

class hr_salary_rule(osv.osv):
    
    _inherit = 'hr.salary.rule'
    
    _columns = {
        'employee': fields.boolean('Employee'),
        'employer': fields.boolean('Employer'),
        'hour': fields.boolean('Hour'),
        'expense': fields.boolean('Expense'),
        'paid_holiday': fields.boolean('Paid holiday'),
        'advance': fields.boolean('Advance'),
        'indemnity': fields.boolean('Indemnity'),
        'indemnity_gross': fields.boolean('Indemnity on gross'),
        'name_to_print_on_line': fields.char('Name to print', size=64),
    }

    _defaults = {
        'employee' : False,
        'employer' : False,
        'hour' : False,
        'expense' : False,
        'paid_holiday' : False,
        'advance' : False,
        'indemnity' : False,
        'indemnity_gross' :  False,
    }
    
hr_salary_rule()

class account_cotisation(osv.osv):
    _description = 'Cotisations'
    _name = 'account.cotisation'
    _columns = {
        'name' : fields.char('Name', size=64),
        'name_to_print_on_line' : fields.char('Name to print', size=64),
        'employee_rate' : fields.float('Employee rate'),
        'employer_rate' : fields.float('Employer rate'),
        'credit_account' : fields.many2one('account.account', 'Third account to credit'),
        'debit_account' : fields.many2one('account.account', 'Account to debit'),
        'account_tax_id' : fields.many2one('account.tax.code', 'Tax Code'),
        'base' : fields.selection(SELECTION_LIST, 'Base'),
        'deductible' : fields.boolean('Deduce'),
        'company_id': fields.many2one('res.company', 'Company'),
      }

    _defaults = {
        'deductible' : True,
    }

account_cotisation()

class account_salary_config(osv.osv):
    _name = 'account.salary.config'
    _description = 'salary.config'
    _columns = {
        'name': fields.char('Name', size=30, required=True),
        'value': fields.float('Value', digits_compute=dp.get_precision('Payroll Rate'), required=True),
    }

account_salary_config()

class hr_payslip(osv.osv):
    _inherit = 'hr.payslip'
    
    def get_config_dict(self, cr, uid, context=None):
        account_config_obj = self.pool.get('account.salary.config')
        account_config_dict = {}
        model_data_obj = self.pool.get('ir.model.data')

        csg_categ = model_data_obj._get_id(cr, uid, 'hr_payroll_configuration', 'config_csg')
        account_config_id = model_data_obj.browse(cr, uid, csg_categ).res_id
        account_config_dict['CSG'] = account_config_obj.browse(cr, uid, account_config_id, context=context)

        ss_categ = model_data_obj._get_id(cr, uid, 'hr_payroll_configuration', 'config_basess')
        account_config_id = model_data_obj.browse(cr, uid, ss_categ).res_id
        account_config_dict['SS'] = account_config_obj.browse(cr, uid, account_config_id, context=context)

        smic_categ = model_data_obj._get_id(cr, uid, 'hr_payroll_configuration', 'config_smic')
        account_config_id = model_data_obj.browse(cr, uid, smic_categ).res_id
        account_config_dict['SMIC'] = account_config_obj.browse(cr, uid, account_config_id, context=context)
        
        return account_config_dict

    def get_payslip_lines(self, cr, uid, contract_ids, payslip_id, context):
        def _sum_salary_rule_category(localdict, category, amount):
            if category.parent_id:
                localdict = _sum_salary_rule_category(localdict, category.parent_id, amount)
            localdict['categories'].dict[category.code] = category.code in localdict['categories'].dict and localdict['categories'].dict[category.code] + amount or amount
            return localdict

        class BrowsableObject(object):
            def __init__(self, pool, cr, uid, employee_id, dict):
                self.pool = pool
                self.cr = cr
                self.uid = uid
                self.employee_id = employee_id
                self.dict = dict

            def __getattr__(self, attr):
                return attr in self.dict and self.dict.__getitem__(attr) or 0.0

        class InputLine(BrowsableObject):
            """a class that will be used into the python code, mainly for usability purposes"""
            def sum(self, code, from_date, to_date=None):
                if to_date is None:
                    to_date = datetime.now().strftime('%Y-%m-%d')
                result = 0.0
                self.cr.execute("SELECT sum(amount) as sum\
                            FROM hr_payslip as hp, hr_payslip_input as pi \
                            WHERE hp.employee_id = %s AND hp.state = 'done' \
                            AND hp.date_from >= %s AND hp.date_to <= %s AND hp.id = pi.payslip_id AND pi.code = %s",
                           (self.employee_id, from_date, to_date, code))
                res = self.cr.fetchone()[0]
                return res or 0.0

        class WorkedDays(BrowsableObject):
            """a class that will be used into the python code, mainly for usability purposes"""
            def _sum(self, code, from_date, to_date=None):
                if to_date is None:
                    to_date = datetime.now().strftime('%Y-%m-%d')
                result = 0.0
                self.cr.execute("SELECT sum(number_of_days) as number_of_days, sum(number_of_hours) as number_of_hours\
                            FROM hr_payslip as hp, hr_payslip_worked_days as pi \
                            WHERE hp.employee_id = %s AND hp.state = 'done'\
                            AND hp.date_from >= %s AND hp.date_to <= %s AND hp.id = pi.payslip_id AND pi.code = %s",
                           (self.employee_id, from_date, to_date, code))
                return self.cr.fetchone()

            def sum(self, code, from_date, to_date=None):
                res = self._sum(code, from_date, to_date)
                return res and res[0] or 0.0

            def sum_hours(self, code, from_date, to_date=None):
                res = self._sum(code, from_date, to_date)
                return res and res[1] or 0.0

        class Payslips(BrowsableObject):
            """a class that will be used into the python code, mainly for usability purposes"""

            def sum(self, code, from_date, to_date=None):
                if to_date is None:
                    to_date = datetime.now().strftime('%Y-%m-%d')
                self.cr.execute("SELECT sum(case when hp.credit_note = False then (pl.total) else (-pl.total) end)\
                            FROM hr_payslip as hp, hr_payslip_line as pl \
                            WHERE hp.employee_id = %s AND hp.state = 'done' \
                            AND hp.date_from >= %s AND hp.date_to <= %s AND hp.id = pl.slip_id AND pl.code = %s",
                            (self.employee_id, from_date, to_date, code))
                res = self.cr.fetchone()
                return res and res[0] or 0.0

        #we keep a dict with the result because a value can be overwritten by another rule with the same code
        result_dict = {}
        rules = {}
        categories_dict = {}

        blacklist = []
        payslip_obj = self.pool.get('hr.payslip')
        inputs_obj = self.pool.get('hr.payslip.worked_days')
        obj_rule = self.pool.get('hr.salary.rule')
        payslip = payslip_obj.browse(cr, uid, payslip_id, context=context)
        worked_days = {}
        for worked_days_line in payslip.worked_days_line_ids:
            worked_days[worked_days_line.code] = worked_days_line
        inputs = {}
        for input_line in payslip.input_line_ids:
            inputs[input_line.code] = input_line

        # Start Custom Code
        account_config_dict = self.get_config_dict(cr, uid, context=context)

        categories_obj = BrowsableObject(self.pool, cr, uid, payslip.employee_id.id, categories_dict)
        input_obj = InputLine(self.pool, cr, uid, payslip.employee_id.id, inputs)
        worked_days_obj = WorkedDays(self.pool, cr, uid, payslip.employee_id.id, worked_days)
        payslip_obj = Payslips(self.pool, cr, uid, payslip.employee_id.id, payslip)
        rules_obj = BrowsableObject(self.pool, cr, uid, payslip.employee_id.id, rules)
        account_config_obj = BrowsableObject(self.pool, cr, uid, payslip.employee_id.id, account_config_dict)

        localdict = {'categories': categories_obj, 'rules': rules_obj, 'payslip': payslip_obj, 'worked_days': worked_days_obj, 'inputs': input_obj, 'config': account_config_obj}
        #get the ids of the structures on the contracts and their parent id as well
        structure_ids = self.pool.get('hr.contract').get_all_structures(cr, uid, contract_ids, context=context)
        #get the rules of the structure and thier children
        rule_ids = self.pool.get('hr.payroll.structure').get_all_rules(cr, uid, structure_ids, context=context)
        #run the rules by sequence
        sorted_rule_ids = [id for id, sequence in sorted(rule_ids, key=lambda x:x[1])]

        for contract in self.pool.get('hr.contract').browse(cr, uid, contract_ids, context=context):
            employee = contract.employee_id
            localdict.update({'employee': employee, 'contract': contract})
            for rule in obj_rule.browse(cr, uid, sorted_rule_ids, context=context):
                key = rule.code + '-' + str(contract.id)
                localdict['result'] = None
                localdict['result_qty'] = 1.0
                #check if the rule can be applied
                if obj_rule.satisfy_condition(cr, uid, rule.id, localdict, context=context) and rule.id not in blacklist:
                    #compute the amount of the rule
                    amount, qty, rate = obj_rule.compute_rule(cr, uid, rule.id, localdict, context=context)
                    #check if there is already a rule computed with that code
                    previous_amount = rule.code in localdict and localdict[rule.code] or 0.0
                    #set/overwrite the amount computed for this rule in the localdict
                    tot_rule = amount * qty * rate / 100.0
                    localdict[rule.code] = tot_rule
                    rules[rule.code] = rule
                    #sum the amount for its salary category
                    localdict = _sum_salary_rule_category(localdict, rule.category_id, tot_rule - previous_amount)
                    #create/overwrite the rule in the temporary results
                    result_dict[key] = {
                        'salary_rule_id': rule.id,
                        'contract_id': contract.id,
                        'name': rule.name,
                        'code': rule.code,
                        'category_id': rule.category_id.id,
                        'sequence': rule.sequence,
                        'appears_on_payslip': rule.appears_on_payslip,
                        'condition_select': rule.condition_select,
                        'condition_python': rule.condition_python,
                        'condition_range': rule.condition_range,
                        'condition_range_min': rule.condition_range_min,
                        'condition_range_max': rule.condition_range_max,
                        'amount_select': rule.amount_select,
                        'amount_fix': rule.amount_fix,
                        'amount_python_compute': rule.amount_python_compute,
                        'amount_percentage': rule.amount_percentage,
                        'amount_percentage_base': rule.amount_percentage_base,
                        'register_id': rule.register_id.id,
                        'amount': amount,
                        'employee_id': contract.employee_id.id,
                        'quantity': qty,
                        'rate': rate,
                    }
                else:
                    #blacklist this rule and its children
                    blacklist += [id for id, seq in self.pool.get('hr.salary.rule')._recursive_search_of_rules(cr, uid, [rule], context=context)]

        result = [value for code, value in result_dict.items()]
        return result

hr_payslip()

class account_salary_type(osv.osv):
    _description = 'Salary type'
    _name = 'account.salary.type'
    _columns = {
        'name': fields.char('Name',size=64),
        'hours_a_week': fields.float('Hours worked a week'),
        'sup1_rate': fields.float('Overtime 1 rate'),
        'sup2_rate': fields.float('Overtime 2 rate'),
        'holiday': fields.boolean('Holidays'),
        'holiday_rate': fields.float('Holidays rate'),
        'brut_account': fields.many2one('account.account', 'Gross salary credit account', required=True),
        'debit_hours_account': fields.many2one('account.account', 'Gross salary debit account', required=True),
        'debit_holiday_account': fields.many2one('account.account', 'Holiday debit account'),
        'tva': fields.many2one('account.tax', 'Tax'),
        'tva_applied': fields.boolean('Taxe applied of cotisations'),
        'cotisation': fields.many2many('account.cotisation', 'salary_cotisation', 'cotisation_id', 'salary_type_id', 'Cotisations'),
        'net_salary_rate': fields.float('Gross - Net rate', digits=(16,4)),
        'holiday_a_month': fields.float('Holidays acquired a month'),
        'debit_acc_tepa_id': fields.many2one('account.account', 'Debit account for the TEPA reduction'),
        'company_id': fields.many2one('res.company', 'Company', required=True),
    }

    _defaults = {
        'sup1_rate': 1.25,
        'sup2_rate': 1.5,
        'company_id': lambda self,cr,uid,c: self.pool.get('res.users').browse(cr, uid, uid, c).company_id.id,
    }

account_salary_type()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
