# -*- coding: utf-8 -*-
#################################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2012 Julius Network Solutions SARL <contact@julius.fr>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#################################################################################

from osv import osv, fields
from tools.translate import _

class hr_payroll_configuration(osv.osv_memory):
    _inherit = 'hr.payroll.configuration'
    _columns = {
    }

    def get_categories(self, cr, uid, context=None):
        if context is None:
            context = {}

        categories_dict = super(hr_payroll_configuration ,self).get_categories(cr, uid, context=context)
        model_data_obj = self.pool.get('ir.model.data')
        
        reduction_int_categ = model_data_obj._get_id(cr, uid, 'hr_payroll_configuration_l10n_fr', 'REDUCTION_INT')
        reduction_int_categ_id = model_data_obj.browse(cr, uid, reduction_int_categ).res_id
        categories_dict['reduction_int_categ'] = reduction_int_categ_id
        
        reduction_rate_categ = model_data_obj._get_id(cr, uid, 'hr_payroll_configuration_l10n_fr', 'REDUCTION_RATE')
        reduction_rate_categ_id = model_data_obj.browse(cr, uid, reduction_rate_categ).res_id
        categories_dict['reduction_rate_categ'] = reduction_rate_categ_id
        
        reduction_categ = model_data_obj._get_id(cr, uid, 'hr_payroll_configuration_l10n_fr', 'REDUCTION')
        reduction_categ_id = model_data_obj.browse(cr, uid, reduction_categ).res_id
        categories_dict['reduction_categ'] = reduction_categ_id
        
        brut025_categ = model_data_obj._get_id(cr, uid, 'hr_payroll_configuration_l10n_fr', '025BRUT')
        brut025_categ_id = model_data_obj.browse(cr, uid, brut025_categ).res_id
        categories_dict['brut025_categ'] = brut025_categ_id
        
        brut025_employee_categ = model_data_obj._get_id(cr, uid, 'hr_payroll_configuration_l10n_fr', '025BRUT_EMPLOYEE')
        brut025_employee_categ_id = model_data_obj.browse(cr, uid, brut025_employee_categ).res_id
        categories_dict['brut025_employee_categ'] = brut025_employee_categ_id
        
        brut025_employer_categ = model_data_obj._get_id(cr, uid, 'hr_payroll_configuration_l10n_fr', '025BRUT_EMPLOYER')
        brut025_employer_categ_id = model_data_obj.browse(cr, uid, brut025_employer_categ).res_id
        categories_dict['brut025_employer_categ'] = brut025_employer_categ_id

        return categories_dict

    def create_reduction_rule(self, cr, uid, data, rule_categories, seq, context=None):
        if context is None:
            context = {}

        rule_obj = self.pool.get('hr.salary.rule')
        account_obj = self.pool.get('account.account')

        categ_int_id = rule_categories['reduction_int_categ']
        categ_rate_id = rule_categories['reduction_rate_categ']
        categ_id = rule_categories['reduction_categ']
        account_debit_id = False
        account_credit_id = False
        account_tax_id = False

        if data.debit_account:
            account_debit_ids = account_obj.search(cr, uid, [('code', '=', data.debit_account.code)])
            if account_debit_ids:
                account_debit_id = account_debit_ids[0]

        if data.credit_account:
            account_credit_ids = account_obj.search(cr, uid, [('code', '=', data.credit_account.code)])
            if account_credit_ids:
                account_credit_id = account_credit_ids[0]

        if data.account_tax_id:
            account_tax_id = data.account_tax_id.id

        rule_obj.create(cr, uid, {
            'name': data.name,
            'code': 'REDUCTION_INT',
            'name_to_print_on_line': data.name_to_print_on_line,
            'appears_on_payslip': False,
            'category_id': categ_int_id,
            'sequence': 500,
            'condition_select': 'python',
            'condition_python': 'result = contract.wage < (1.6 * config.SMIC.value) and categories.BASIC',
            'amount_select': 'code',
            'amount_python_compute': 'result = ((config.REDUCTION.value / 100) / 0.60) * (1.6 * ((config.SMIC.value * 1.1 * categories.TOTAL_NB) / categories.BASIC) - 1) * contract.wage * 1.1 * categories.TOTAL_NB',
        })
        
        rule_obj.create(cr, uid, {
            'name': data.name,
            'code': 'REDUCTION_RATE',
            'name_to_print_on_line': data.name_to_print_on_line,
            'appears_on_payslip': False,
            'category_id': categ_rate_id,
            'sequence': 501,
            'condition_select': 'python',
            'condition_python': 'result = categories.REDUCTION_INT',
            'amount_select': 'code',
            'amount_python_compute': 'result_qty = 1\nresult_rate = 100\nif categories.REDUCTION_INT > (config.REDUCTION.value * categories.BASIC / 100): \
\n    result = - config.REDUCTION.value\
\nelse:\n    result = - 100 * categories.REDUCTION_INT / categories.BASIC',
        })
        
        rule_obj.create(cr, uid, {
            'name': data.name,
            'code': 'REDUCTION',
            'name_to_print_on_line': data.name_to_print_on_line,
            'appears_on_payslip': True,
            'category_id': categ_id,
            'sequence': 502,
            'condition_select': 'python',
            'condition_python': 'result = categories.REDUCTION_INT',
            'amount_select': 'code',
            'amount_python_compute': 'result_qty = 1\nresult = categories.BASIC\nresult_rate = categories.REDUCTION_RATE',
            'employer': True,
            'account_debit': account_debit_id,
            'account_credit': account_credit_id,
            'account_tax_id': account_tax_id,
        })
        
        return seq

    def create_025brut_rule(self, cr, uid, data, rule_categories, seq, context=None):
        if context is None:
            context = {}

        rule_obj = self.pool.get('hr.salary.rule')
        account_obj = self.pool.get('account.account')

        account_debit_id = False
        account_credit_id = False
        account_tax_id = False

        if data.deductible:
            categ_id = rule_categories['ded_categ']
            employee_categ_id = rule_categories['brut025_employee_categ']
            employer_categ_id = rule_categories['brut025_employer_categ']
        else:
            categ_id = rule_categories['net_categ']
            employee_categ_id = rule_categories['brut025_employee_categ']
            employer_categ_id = rule_categories['brut025_employer_categ']

        if data.debit_account:
            account_debit_ids = account_obj.search(cr, uid, [('code', '=', data.debit_account.code)])
            if account_debit_ids:
                account_debit_id = account_debit_ids[0]

        if data.credit_account:
            account_credit_ids = account_obj.search(cr, uid, [('code', '=', data.credit_account.code)])
            if account_credit_ids:
                account_credit_id = account_credit_ids[0]

        if data.account_tax_id:
            account_tax_id = data.account_tax_id.id

        if data.employee_rate:
            employee_rule_id = rule_obj.create(cr, uid, {
                'name': data.name,
                'code': self.update_code(data.name_to_print_on_line,_('(Employee)')),
                'name_to_print_on_line': data.name_to_print_on_line,
                'category_id': employee_categ_id,
                'sequence': seq,
                'condition_select': 'none',
                'amount_select': 'percentage',
                'amount_percentage_base': 'categories.025BRUT',
                'amount_percentage': data.employee_rate,
                'account_debit': False,
                'account_credit': account_credit_id,
                'account_tax_id': account_tax_id,
                'employee': True,
            })
            seq += 1

        if data.employer_rate:
            employer_rule_id = rule_obj.create(cr, uid, {
                'name': data.name,
                'code': self.update_code(data.name_to_print_on_line,_('(Employer)')),
                'name_to_print_on_line': data.name_to_print_on_line,
                'category_id': employer_categ_id,
                'sequence': seq,
                'condition_select': 'none',
                'amount_select': 'percentage',
                'amount_percentage_base': 'categories.BRUT025',
                'amount_percentage': data.employer_rate,
                'account_debit': account_debit_id,
                'account_credit': account_credit_id,
                'account_tax_id': account_tax_id,
                'employer': True,
            })
            seq += 1
        return seq
    
    def create_rules(self, cr, uid, data, rule_categories, seq, context=None):
        seq = super(hr_payroll_configuration, self).create_rules(cr, uid, data, rule_categories, seq, context=context)
        if data.base == 'reduction':
            seq = self.create_reduction_rule(cr, uid, data, rule_categories, seq, context=context)
        if data.base == '025brut':
            seq = self.create_025brut_rule(cr, uid, data, rule_categories, seq, context=context)
        return seq
    
    def get_default_rules(self, cr, uid, salary_data, context=None):
        res = super(hr_payroll_configuration, self).get_default_rules(cr, uid, salary_data, context=context)
        model_data_obj = self.pool.get('ir.model.data')
        brut025_rule = model_data_obj._get_id(cr, uid, 'hr_payroll_configuration_l10n_fr', 'hr_configuration_rule_025BRUT')
        brut025_rule_id = model_data_obj.browse(cr, uid, brut025_rule).res_id
        res.append(brut025_rule_id)
        return res

hr_payroll_configuration()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
